/**
 * Victor Silva - University of Bridgeport 20 Feb. 1997
 *
 * Use this class by calling HLS.fromRGB to convert an
 * int array containing image pixels into HLS color space.
 *
 * HLS and RGB Converter class.  This class allows conversion
 * between RGB and HLS (Hue, Lightness and Saturation).
 * The conversion algorithm is from Foley and van Dam
 * "Computer Graphics".
 *
 * Input is int[] one for each R, G, B.
 * Output is int[] ARGB format.
 *
 */
package VS;

import java.awt.*;
import java.io.*;
import ipl.*;

public class HLS extends ColorConverter {


   private ColorUtils CU = new ColorUtils();
   
   // scale factor is used to brighten up the image.
   private double sf = 1.0;
 
public HLS(ProcessPlane pp_) {
  	super(pp_);
}


 public int[] fromRGB() {
 	  int totalPix = fi.getLength();
      int[] pixArray = new int[totalPix];
      float max = 0f;
      float min = 255f;
      double H;
      double L;
      double S;
      double delta;
      

      for (int i=0; i<totalPix; i++) {
         max = fi.max(i);
         min = fi.min(i);

         L = ((double)(max + min)) / 2.0;
         // H is really undefined.
         H = 0.0;
         S = 0.0;

         // Calculate saturation
         // Achromatic case
         if (max == min) {
            fi.r[i] = (float)H;
            fi.g[i] = (float)L;
            fi.b[i] = (float)S;

            pixArray[i] = CU.putRGB((short)H, (short)L, (short)S);
         }
         else
         {
            delta = ((double)(max - min));
            // Calculate saturation
            S = (L <= 0.5) ? (delta / (double)(max + min)) : (delta / (2.0 -
               (double)(max + min)));

            // Calculate hue.
            if (fi.r[i] == max) {
               H = ((double)(fi.g[i] - fi.b[i])) / delta;
            }
            else {
               if (fi.g[i] == max)
                  H = 2.0 + ((double)(fi.b[i] - fi.r[i])) / delta;
               else if (fi.b[i] == max)
                     H = 4.0 + ((double)(fi.r[i] - fi.g[i])) / delta;
            }
            H *= 60.0;
            // Degrees non-negative.
            if (H < 0.0)
               H += 360.0;

            fi.r[i] = (float)H;
            fi.g[i] = (float)L;
            fi.b[i] = (float)S;

            pixArray[i] = CU.putRGB((short)H, (short)L, (short)S);
         } // end Chromatic case
      }

      return(pixArray);
   }

public float getMinIntensity() {
	float minI = 99999;
	for (int i=0; i < fi.r.length; i++) 
		if (minI > fi.g[i]) 
			minI =  fi.g[i];
	return minI;
}

public float getMaxIntensity() {
	float maxI = -99999;
	for (int i=0; i < fi.g.length; i++) 
		if (maxI < fi.g[i]) 
			maxI = fi.g[i];
	return maxI;
}

private int xfer(double f) {
	double sf = 31.875 / Math.log(2.0);
	return ((int) (sf * Math.log(f + 1))); 
}

public void equalize() {
	for (int i = 0; i < fi.g.length; i++) {
		fi.g[i] = xfer(fi.g[i]);
	}

}

// assume that luminance is stored in HLS_green[i]
public void autoScale() {
	float max = getMaxIntensity();
	float min = getMinIntensity();
	System.out.println("Max luminance = " + max);
	System.out.println("Min luminance = " + min);
	float sf = (float) (255.0 / (max - min));
	System.out.println("Scaling in luminance by" + sf);
	for (int i = 0; i < fi.g.length; i++) {
		fi.g[i] *= sf;
	}
}
 public void brighten(double sf_) {
	sf = sf_;
  }
   public int[] toRGB()  {
   	float r[];
   	float g[];
   	float b[];
   	if (fi == null) 
   		System.out.println("ER!: HLS, fi == null");
   	r = fi.r;
   	g = fi.g;
   	b = fi.b;
     int totalPix = r.length;

      double m1, m2;
      double H, L, S;
      double R, G, B;

      for (int i=0; i<totalPix; i++)
      {
         H = (double)r[i];
         L = (double)sf * g[i];
         S = (double)b[i];

         m2 = (L <= 0.5) ? (L * (L + S)) : (L + S - (L * S));
         m1 = 2.0 * L - m2;

         if (S == 0.0)
         {
            pp.setPixel(i, (int) L, (int) L, (int) L, 255);
         }
         else
         {
            R = Value(m1, m2, H + 120.0);
            G = Value(m1, m2, H);
            B = Value(m1, m2, H - 120.0);

            r[i] = (short)R;
            g[i] = (short)G;
            b[i] = (short)B;
            pp.setPixel(i, (int) R, (int) G, (int) B, 255);
         }
      }

      return(pp.pels);
   }

   double Value(double n1, double n2, double hue)
   {
      if (hue > 360.0)
      {
         hue -= 360.0;
      }
      else
      {
         if (hue < 0.0)
         {
            hue += 360.0;
         }
      }

      if (hue < 60.0)
      {
         return(n1 + ((n2 - n1) * hue) / 60.0);
      }
      else
      {
         if (hue < 180.0)
         {
            return(n2);
         }
         else
         {
            if (hue < 240.0)
            {
               return(n1 + ((n2 - n1) * (240.0 - hue)) / 60.0);
            }
            else
            {
               return(n1);
            }
         }
      }
   }

}

